// SSM_Pathmanagement.h: interface for the SSM_Pathmanagement class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SSM_PATHMANAGEMENT_H__B258FDD4_7AC0_4669_A5B5_7BA6E6608E11__INCLUDED_)
#define AFX_SSM_PATHMANAGEMENT_H__B258FDD4_7AC0_4669_A5B5_7BA6E6608E11__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include"globals.h"

/* The states of pathmanagement, also used for network status change */
#define  PM_ACTIVE              0
#define  PM_INACTIVE            1
#define  PM_ADDED               2
#define  PM_REMOVED             3

#define  PM_PATH_UNCONFIRMED    5


#define PM_INITIAL_HB_INTERVAL  30000

class SSM_Pathmanagement  
{
public:
	SSM_Pathmanagement();
	virtual ~SSM_Pathmanagement();



	
	/******************** Functions to answer peer HB requests ****************************************/
	/**
	 * Function is called to perform a on demand HB on a certain path
	 */
	static int pm_doHB(gshort pathID);


	/* pm_heartbeat is called when a heartbeat was received from the peer.
	   params: heartbeatChunk: the heartbeat chunk
	*/
	static void pm_heartbeat(SCTP_heartbeat * heartbeatChunk, unsigned int source_address);


	/******************** Signals *********************************************************************/

	/*------------------- Signals from the Unix-Interface --------------------------------------------*/

	/* pm_heartbeatTimer is called by the adaption-layer when the heartbeat timer expires.
	   params: timerID:            ID of timer
			   associationIDvoid:  pointer to the association-ID
			   pathIDvoid:         pointer to the path-ID
	*/
	static void pm_heartbeatTimer(TimerID timerID, void *associationIDvoid, void *pathIDvoid);



	/* pm_heartbeatAck is called when a heartbeat acknowledgement was received from the peer.
	   params: heartbeatChunk: the heartbeat chunk
	*/
	static void pm_heartbeatAck(SCTP_heartbeat * heartbeatChunk);

	/*------------------- Signals from SCTP internal modules -----------------------------------------*/

	/* pm_chunksAcked is called by reliable transfer whenever chunks have been acknowledged. 
	   Params: pathID:      path-ID
			   newRTO:      the newly determined RTO in milliseconds
							newRTO = 0 ==> no RTO measurements done
	*/
	static void pm_chunksAcked(short pathID, unsigned int newRTO);

	/**
	 * function to be called every time we send data to a path
	 * to keep it from becoming "idle"
	 */
	static void pm_chunksSentOn(short pathID);


	/* pm_chunksRetransmitted is called by reliable transfer whenever chunks have been 
	   retransmitted.
	   Params: pathID:      path-ID 
	*/
	static gboolean pm_chunksRetransmitted(short pathID);



	/* pm_rto_backoff is called by reliable transfer when the T3 retransmission timer expires.
	   Each call of this function doubles the RTO (timer back off).
	   Params: pathID:      path-ID 
	*/
	static void pm_rto_backoff(short pathID);

	/*------------------- Helper Function  ----------------------------------------------------------*/

	static unsigned int pm_getTime(void);


	/*------------------- Functions called by the ULP ------------------------------------------------*/


	/* pm_enableHB is called when ULP wants to enable heartbeat.
	*/
	static int pm_enableHB(short pathID, unsigned int hearbeatIntervall);



	/* pm_disableAllHB is called when on shutdown to disable all heartbeats.
	*/
	static void pm_disableAllHB(void);



	/* pm_disableHB is called when ULP wants to disable heartbeat.
	*/
	static int pm_disableHB(short pathID);




	/* pm_setPrimaryPath sets the primary path.
	   Params: primaryPathID:      path-ID
	*/
	static short pm_setPrimaryPath(short pathID);

	static int  pm_getMaxPathRetransmisions(void);

	static int  pm_setRtoInitial(int new_rto_initial);

	static int  pm_getRtoInitial(void);


	static int  pm_setRtoMin(int new_rto_min);

	static int  pm_getRtoMin(void);

	static int  pm_setRtoMax(int new_rto_max);

	static int  pm_getRtoMax(void);

	static int  pm_setHBInterval(unsigned int new_interval);

	static int pm_getHBInterval(short pathID, unsigned int* current_interval);


	static int  pm_setMaxPathRetransmisions(int new_max);

	/*------------------- Functions called by ULP to read pathmanagement state info ------------------*/


	/**
	 * pm_readRTO returns the currently set RTO value in msecs for a certain path.
	 * @param     pathID      index of the address/path
	 * @return                path's current RTO in msecs
	*/
	static unsigned int pm_readRTO(short pathID);



	/* pm_readSRTT is called by reliable transfer and sctp-control to adjust T3-timeout and
	   init-timeout, respectively.
	   Params: pathID:      path-ID 
			   returns:     current smoothed round trip time or 0xffffffff on error
	*/
	static unsigned int pm_readSRTT(short pathID);


	static unsigned int pm_readRttVar(short pathID);


	/**
	 * pm_readState returns the current state of the path.
	 * @params pathID      index of the path that is checked for its state
	 * @return  state value for this path (PM_ACTIVE, PM_INACTIVE, PM_PATH_UNCONFIRMED)
	 */
	static short pm_readState(short pathID);



	/* pm_readPrimaryPath returns the primary path.
	   Params: returns      primary path, or 0xFFFF as error
	*/
	static unsigned short pm_readPrimaryPath(void);



	/*------------------- Functions called to create, init and delete pathman-instances --------------*/

	/* pm_setPaths modufies number of paths and sets the primary path.
	   This is required for association setup, where the local ULP provides
	   only one path and the peer may provide additional paths.
	   This function also initializes the path structures and starts the heartbeat timer for each
	   path. For this reason it is recommended to call this function when communication up is called.
	   Params: primaryPathID:      path-ID
			   noOfPaths           number of paths
	*/
	static short pm_setPaths(short noOfPaths, short primaryPathID);



	/* pm_newPathman creates a new instance of pathmanagement. There is one pathmanagement instance
	   par association.
	   params: numberOfPaths:    # of paths of the association.
			   primaryPath:      initial primary path.
	*/
	static void *pm_newPathman(short numberOfPaths, short primaryPath, void* sctpInstance);



	/* Deletes the instance pointed to by pathmanPtr.
	*/
	static void pm_deletePathman(void *pathmanPtr);


};

#endif // !defined(AFX_SSM_PATHMANAGEMENT_H__B258FDD4_7AC0_4669_A5B5_7BA6E6608E11__INCLUDED_)
