// SSM_SCTP_Control.h: interface for the SSM_SCTP_Control class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SSM_SCTP_CONTROL_H__713E57E5_8439_4920_AF17_9506C99E0B4C__INCLUDED_)
#define AFX_SSM_SCTP_CONTROL_H__713E57E5_8439_4920_AF17_9506C99E0B4C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "globals.h"
#include "messages.h"
#include "sctp.h"

/* The states of SCTP-control */
#define CLOSED           0
#define COOKIE_WAIT      1
#define COOKIE_ECHOED      2
#define ESTABLISHED      3
#define SHUTDOWNPENDING  4
#define SHUTDOWNRECEIVED 5
#define SHUTDOWNSENT     6
#define SHUTDOWNACKSENT  7

/* Return codes for a number of functions that treat incoming chunks */
/* these are used in the rbundling module !                          */
#define STATE_STOP_PARSING_REMOVED  -1
#define STATE_OK        0
#define STATE_STOP_PARSING  1


class SSM_SCTP_Control  
{
public:
	SSM_SCTP_Control();
	virtual ~SSM_SCTP_Control();



	static void scu_associate(unsigned short noOfOutStreams,
                   unsigned short noOfInStreams,
                   union sockunion* destinationList,
                   unsigned int numDestAddresses,
                   gboolean withPRSCTP);



	/* initiates the shutdown of this association.
	*/
	static void scu_shutdown(void);



	/* aborts this association.
	*/
	static void scu_abort(short error_type, unsigned short error_param_length, unsigned char* error_param_data);



	/*------------------- Functions called by the (de-)bundling for recieved control chunks ----------*/

	/* sctlr_init is called by bundling when a init message is received from the peer.
	   New data must not be allocated for this new association.
	   The following data are created and included in the init acknowledgement:
	   - cookie: contains init-data, local tag, initial TSN, # of send streams, # of receive streams
		 and a signature.
	   - local tag: randomly generated, is included in the cookie and the initiate tag field of the
					init acknowledgement.
	   - inititial TSN: randomly generated, is included in the cookie and the initial TSN field of the 
						init acknowledgement.
	   Params: init:    data of init-chunk including optional parameters without chunk header
	*/
	static int sctlr_init(SCTP_init * init);



	/* sctlr_initAck is called by bundling when a init acknowledgement was received from the peer.
	   The following data are retrieved from the init-data and saved for this association:
	   - remote tag from the initiate tag field
	   - receiver window credit of the peer
	   - # of send streams of the peer, must be lower or equal the # of receive streams this host
		 has 'announced' with the init-chunk.
	   - # of receive streams the peer allows the receiver of this initAck to use.
   
	   The initAck must contain a cookie which is returned to the peer with the cookie acknowledgement.

	   Params: initAck: data of initAck-chunk including optional parameters without chunk header
	*/
	static int sctlr_initAck(SCTP_init * initAck);



	/* sctlr_cookie_echo is called by bundling when a cookie chunk was received from  the peer.
	   The following data are retrieved from the cookie and saved for this association:
	   - from the init chunk: + peers tag.
							  + peers receiver window credit.
							  + peers initial TSN.
							  + peers network address list if multihoming is used.
	   - local tag generated before the initAck was sent.
	   - this side initial TSN generated before the initAck was sent.
	   - # of send streams this side uses, must be lower or equal to peers # of receive streams from init
		 chunk
	   - # of receive streams this side uses, can be lower than peers # of send streams the requested in
		 the init chunk.
	*/
	static void sctlr_cookie_echo(SCTP_cookie_echo * cookie);



	/* sctlr_cookieAck is called by bundling when a cookieAck chunk was received from  the peer.
	   The only purpose is to inform the active side that peer has received the cookie chunk.
	   The association is established after this function is called.
	   StartOfDataTX is called at Flowcontrol to start transmission of data chunks.
	   The ULP is informed by the communication up notification.
	*/
	static void sctlr_cookieAck(SCTP_simple_chunk * cookieAck);



	/* sctlr_shutdown is called by bundling when a shutdown chunk was received from  the peer.
	   The function initiates a gracefull shutdown of the association.
	   Params: cumulativeTSN_ack: highest consecutive TSN acked.
	*/
	static int sctlr_shutdown(SCTP_simple_chunk * shutdown_chunk);



	/* sctlr_shutdownAck is called by bundling when a shutdownAck chunk was received from  the peer.
	   The function initiates a gracefull shutdown of the association.
	*/
	static int sctlr_shutdownAck(void);

	/* sctlr_shutdownComplete is called by bundling when a shutdownComplete chunk was received from the peer.
	*/
	static int sctlr_shutdownComplete(void);

	/* sctlr_abort is called by bundling when a abort chunk was received from  the peer.
	   The association is terminated imediately.
	*/
	static int sctlr_abort(void);



	/* sctlr_staleCookie is called by bundling when a error chunk with cause 'stale cookie'
	   was received from  the peer.
	   Params: staleness: microseconds the cookie life time was exceeded.
	*/
	static void sctlr_staleCookie(SCTP_simple_chunk * error_chunk);



	/*------------------- Functions called by reliable transfer --------------------------------------*/

	/* This function is called by reliable transfer if all sent chunks in its retransmission queue have
	   been acked. 
	*/
	static void sci_allChunksAcked(void);



	/*------------------- Functions called message by distribution to create and delete --------------*/

	/* newSCTP_control allocates data for a new SCTP-Control instance
	*/
	static void *sci_newSCTP_control(void* sctpInstance);



	/* deleteSCTP_control allocates data for a new SCTP-Control instance
	*/
	static void sci_deleteSCTP_control(void *sctpControlData);

	/**
	 * function returns the state of the current SCTP instance
	 */
	static guint32 sci_getState(void);


	static int sci_getMaxAssocRetransmissions(void);

	static int sci_getMaxInitRetransmissions(void);

	static int sci_getCookieLifeTime(void);

	static int sci_setMaxAssocRetransmissions(int new_max);

	static int sci_setMaxInitRetransmissions(int new_max);

	static int sci_setCookieLifeTime(int new_max);

	/**
	 * function returns true if state of the association
	 * is in the SHUTDOWNs, else FALSE
	 */

	static gboolean sci_shutdown_procedure_started(void);


};

#endif // !defined(AFX_SSM_SCTP_CONTROL_H__713E57E5_8439_4920_AF17_9506C99E0B4C__INCLUDED_)
